<?php

namespace App\Http\Controllers\Voyager;

use App\Jobs\SyncCommandsToPos;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use TCG\Voyager\Database\Schema\SchemaManager;
use TCG\Voyager\Events\BreadDataAdded;
use TCG\Voyager\Events\BreadDataDeleted;
use TCG\Voyager\Events\BreadDataRestored;
use TCG\Voyager\Events\BreadDataUpdated;
use TCG\Voyager\Events\BreadImagesDeleted;
use Carbon\Carbon;
use Voyager;
use Illuminate\Support\Facades\Redirect;
use App\Mail\ValidatedCommandMail;
use App\Models\DeliveryState;
use App\Models\StockOperationProduct;
use App\Models\TransportCompany;
use TCG\Voyager\Http\Controllers\Traits\BreadRelationshipParser;
use Illuminate\Support\Facades\Artisan;

use App\Repositories\UserRepository;
use App\Repositories\CommandStateRepository;
use App\Repositories\PayementModeRepository;
use App\Repositories\CommandRepository;
use App\Repositories\DeliveryCommandRepository;
use App\Repositories\MagasinRepository;
use App\Repositories\StockOperationRepository;
use App\Repositories\StockOperationProductRepository;
use App\Repositories\StockOperationTypeRepository;
use App\Repositories\CommandPriceRepository;
use App\Repositories\RoleRepository;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;

class CommandController extends \TCG\Voyager\Http\Controllers\VoyagerBaseController
{
    use BreadRelationshipParser;

    protected $roleRepository;
    protected $userRepository;
    protected $commandStateRepository;
    protected $payementModeRepository;
    protected $commandRepository;
    protected $deliveryCommand;

    protected $stockOperationRepository;
    protected $stockOperationProductRepository;
    protected $stockOperationTypeRepository;
    protected $stockOperationType;
    protected $magasinRepository;
    protected $commandPriceRepository;

    public function __construct(
        UserRepository $userRepository,
        CommandStateRepository $commandStateRepository,
        PayementModeRepository $payementModeRepository,
        CommandRepository $commandRepository,
        DeliveryCommandRepository $deliveryCommandRepository,
        StockOperationRepository $stockOperationRepository,
        StockOperationProductRepository $stockOperationProductRepository,
        StockOperationTypeRepository $stockOperationTypeRepository,
        MagasinRepository $magasinRepository,
        CommandPriceRepository $commandPriceRepository,
        RoleRepository $roleRepository
    ){
        $this->userRepository = $userRepository;
        $this->commandStateRepository = $commandStateRepository;
        $this->payementModeRepository = $payementModeRepository;
        $this->commandRepository = $commandRepository;
        $this->deliveryCommandRepository = $deliveryCommandRepository;

        $this->stockOperationRepository = $stockOperationRepository;
        $this->stockOperationProductRepository = $stockOperationProductRepository;
        $this->stockOperationTypeRepository = $stockOperationTypeRepository;
        $this->magasinRepository = $magasinRepository;
        $this->commandPriceRepository = $commandPriceRepository;
        $this->roleRepository = $roleRepository;
    }

    public function printCommand(Request $request, $commandId) {

        $pdfFile = $this->commandRepository->generatePdfFile($commandId);

        return $pdfFile->stream();

    }

    public function getInvitedCommands(Request $request, $commandId) {

        $commands = $this->commandRepository->all(
            $search = [ "parentId" => $commandId ],
            $skip = null,
            $limit = null,
            $columns = ['*'],
            $with = ["client.user"]
        );

        $commands->each->append("total");

        return response()->json([
            'commands' => $commands,
        ], 200);

    }

    public function changeCommandState(Request $request, $commandId)
    {
        $command = $this->commandRepository->find($commandId);

        $commandState = $this->commandStateRepository->getNextState($command->state->tag);

        $command->stateId = $commandState->id;

        $command->save();

        $command = $this->commandRepository->find($commandId);

        if($command->state->tag == "validated") {

            $type = $this->stockOperationTypeRepository->allQuery([
                "tag" => "vente"
            ])->first();

            $magasinId = $command->magasinId;
            if ($magasinId == null) $magasinId = $this->magasinRepository->getByTag("default")->id;

            $stockOperation = $this->stockOperationRepository->create([
                "magasinId" => $magasinId,
                "userId" => auth()->user()->id,
                "typeId" => $type->id,
                "planifiedFor" => Carbon::parse($command->planifiedFor)->weekday()
            ]);

            $command->prices->map(function($price) use ($stockOperation){
                $price->products->map(function($product) use ($stockOperation, $price){
                    return $this->stockOperationProductRepository->create([
                        "operationId" => $stockOperation->id,
                        "productId" => $product->id,
                        "quantity" => $price->pivot->quantity
                    ]);
                });
            });

            if($command->filePath == null) {
                $pdfFile = $this->commandRepository->generatePdfFile($commandId);
                $command = $this->commandRepository->find($commandId);
            }

            $commandList = collect([$command]);
            if($command->invitedCommands->count() > 0){
                $commandList = $commandList->merge($command->invitedCommands);
            }

            $commandList->map(function($cmd) {
                $message = (new ValidatedCommandMail($cmd))->onQueue('emails');
                $ccEmail = setting('constants.MAIL_CARBON_COPY');
                $email = \Mail::to($cmd->client->user->email);
                if($ccEmail && $ccEmail != "") $email->cc([$ccEmail ? $ccEmail : null]);
                $email->queue($message);
            });

        }

        $command->save();

        return response()->json([
            'commandState' => $commandState,
            'command' => $command
        ], 200);

    }

    public function changeCommandDeliveryMan(Request $request, $commandId, $deliveryManId)
    {

        $deliveryCommand = $this->deliveryCommandRepository->create([
            "userId" => $deliveryManId,
            "commandId" => $commandId
        ]);

        return response()->json([
            'status' => $deliveryCommand
        ], 200);

    }

    //***************************************
    //               ____
    //              |  _ \
    //              | |_) |
    //              |  _ <
    //              | |_) |
    //              |____/
    //
    //      Browse our Data Type (B)READ
    //
    //****************************************

    public function index(Request $request)
    {
        
        return;
    
/*        if(!$request->query('limit')) {
            $query = $request->query();
            $query['limit'] = '100';
    
            return Redirect::to(url()->current() . '?' . http_build_query($query));
    
        }*/
        
        // GET THE SLUG, ex. 'posts', 'pages', etc.
        $slug = $this->getSlug($request);
        
        // GET THE DataType based on the slug
        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();
        
        // Check permission
        $this->authorize('browse', app($dataType->model_name));

        $getter = $dataType->server_side ? 'paginate' : 'get';

        $search = (object) ['value' => $request->get('s'), 'key' => $request->get('key'), 'filter' => $request->get('filter')];
        $searchable = $dataType->server_side ? array_keys(SchemaManager::describeTable(app($dataType->model_name)->getTable())->toArray()) : '';
        $orderBy = $request->get('order_by', $dataType->order_column);
        $sortOrder = $request->get('sort_order', null);
        $usesSoftDeletes = false;
        $showSoftDeleted = false;
        $orderColumn = [];
        if ($orderBy) {
            $index = $dataType->browseRows->where('field', $orderBy)->keys()->first() + 1;
            $orderColumn = [[$index, 'desc']];
            if (!$sortOrder && isset($dataType->order_direction)) {
                $sortOrder = $dataType->order_direction;
                $orderColumn = [[$index, $dataType->order_direction]];
            } else {
                $orderColumn = [[$index, 'desc']];
            }
        }

        // Next Get or Paginate the actual content from the MODEL that corresponds to the slug DataType
        if (strlen($dataType->model_name) != 0) {
            $model = app($dataType->model_name);

            if ($dataType->scope && $dataType->scope != '' && method_exists($model, 'scope'.ucfirst($dataType->scope))) {
                $query = $model->{$dataType->scope}();
            } else {
                $query = $model::select('*');
            }
            
           $state = DeliveryState::where('tag', 'pending')->first();

            if($request->has("stateId") && $request->get("stateId") != "" && $state->id != $request->get("stateId")) {
                if($request->stateId == 'canceled') $query = $query->where('canceled', true);
                else $query = $query->where("delivery_state_id", $request->stateId);
            } else {
                $query = $query->where(function($query) use($state){
                    $query
                        ->whereNull("delivery_state_id")
                        ->orWhere("delivery_state_id", $state->id);
                });
                $query = $query->where('canceled', false);
            }

            $query = $query->whereNull('parentId');

            if($request->fromDate) {
                $query = $query->whereDate("created_at", ">=", $request->fromDate);
            }
            if($request->toDate) {
                $query = $query->whereDate("created_at", "<=", $request->toDate);
            }

            // Use withTrashed() if model uses SoftDeletes and if toggle is selected
            if ($model && in_array(SoftDeletes::class, class_uses($model)) && Auth::user()->can('delete', app($dataType->model_name))) {
                $usesSoftDeletes = true;

                if ($request->get('showSoftDeleted')) {
                    $showSoftDeleted = true;
                    $query = $query->withTrashed();
                }
            }

            // If a column has a relationship associated with it, we do not want to show that field
            $this->removeRelationshipField($dataType, 'browse');

            if ($search->value != '' && $search->key && $search->filter) {
                $search_filter = ($search->filter == 'equals') ? '=' : 'LIKE';
                $search_value = ($search->filter == 'equals') ? $search->value : '%'.$search->value.'%';
                $query->where($search->key, $search_filter, $search_value);
            }

            if ($orderBy && in_array($orderBy, $dataType->fields())) {
                $querySortOrder = (!empty($sortOrder)) ? $sortOrder : 'desc';
                $dataTypeContent = call_user_func([
                    $query->orderBy($orderBy, $querySortOrder),
                    $getter,
                ]);
            } elseif ($model->timestamps) {
                $dataTypeContent = call_user_func([$query->latest($model::CREATED_AT), $getter]);

            } else {
                $dataTypeContent = call_user_func([$query->orderBy($model->getKeyName(), 'DESC'), $getter]);
            }
            
            $limit = $request->query('limit');
            $days = $request->query('days');
            
            if($days) $dataTypeContent = call_user_func([$query->where('created_at', '>=', Carbon::now()->subDays($days)->startOfDay()), $getter]);
            
            if($limit) $dataTypeContent = call_user_func([$query->limit($limit), $getter]);


            // Replace relationships' keys for labels and create READ links if a slug is provided.
            $dataTypeContent = $this->resolveRelations($dataTypeContent, $dataType);
        } else {
            // If Model doesn't exist, get data from table name
            $dataTypeContent = call_user_func([DB::table($dataType->name), $getter]);
            $model = false;
        }

        // Check if BREAD is Translatable
        if (($isModelTranslatable = is_bread_translatable($model))) {
            $dataTypeContent->load('translations');
        }

        // Check if server side pagination is enabled
        $isServerSide = isset($dataType->server_side) && $dataType->server_side;

        // Check if a default search key is set
        $defaultSearchKey = $dataType->default_search_key ?? null;

        $view = 'voyager::bread.browse';

        if (view()->exists("voyager::$slug.browse")) {
            $view = "voyager::$slug.browse";
        }

        $role = $this->roleRepository->getByTag(config('constants.DELIVER_ROLE'));

        $deliveryMen = $this->userRepository->all(
            ["role_id" => $role->id]
        );

        $commandStates = DeliveryState::all();

        return Voyager::view($view, compact(
            'deliveryMen',
            'commandStates',
            'dataType',
            'dataTypeContent',
            'isModelTranslatable',
            'search',
            'orderBy',
            'orderColumn',
            'sortOrder',
            'searchable',
            'isServerSide',
            'defaultSearchKey',
            'usesSoftDeletes',
            'showSoftDeleted'
        ));
    }
    
    
    public function getPaginatedCommands(Request $request) {
        
        $state = $request->state;
        $key = $request->key;
        $pagination = $request->pagination ?? 10;
        
        $commands = \App\Models\Command::with('client', 'address');
        
        if($state == 'canceled') $commands = $commands->where('canceled', true);
        else if($state == 'validated') $commands = $commands->where('validated', true);
        // else $commands = $commands->where('validated', false);
        
        if($key != "all") {
            $commands = $commands
                ->where('trackingNumber', 'like', '%' . $key . '%')
                ->orWhereHas('client', function($query) use($key) {
                    $query->where('phone', 'like', '%' . $key . '%');
                    $query->orWhereHas('user', function($query) use($key) {
                        $query->where('name', 'like', '%' . $key . '%');
                    });
                });
        }
        
        $commands = $commands
            ->latest('created_at')
            ->paginate($pagination);
          
        $view = 'voyager::command.index';
        
        $commands->appends($request->query());
        
        return Voyager::view($view, [
            'dataTypeContent' => $commands,
            'key' => $key && $key != "all" ? $key : ""
        ]);
    }

    //***************************************
    //                _____
    //               |  __ \
    //               | |__) |
    //               |  _  /
    //               | | \ \
    //               |_|  \_\
    //
    //  Read an item of our Data Type B(R)EAD
    //
    //****************************************

    public function show(Request $request, $id)
    {
        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        $isSoftDeleted = false;

        if (strlen($dataType->model_name) != 0) {
            $model = app($dataType->model_name);

            // Use withTrashed() if model uses SoftDeletes and if toggle is selected
            if ($model && in_array(SoftDeletes::class, class_uses($model))) {
                $model = $model->withTrashed();
            }
            if ($dataType->scope && $dataType->scope != '' && method_exists($model, 'scope'.ucfirst($dataType->scope))) {
                $model = $model->{$dataType->scope}();
            }
            $dataTypeContent = call_user_func([$model, 'findOrFail'], $id);
            if ($dataTypeContent->deleted_at) {
                $isSoftDeleted = true;
            }
        } else {
            // If Model doest exist, get data from table name
            $dataTypeContent = DB::table($dataType->name)->where('id', $id)->first();
        }

        // Replace relationships' keys for labels and create READ links if a slug is provided.
        $dataTypeContent = $this->resolveRelations($dataTypeContent, $dataType, true);

        // If a column has a relationship associated with it, we do not want to show that field
        $this->removeRelationshipField($dataType, 'read');

        // Check permission
        $this->authorize('read', $dataTypeContent);

        // Check if BREAD is Translatable
        $isModelTranslatable = is_bread_translatable($dataTypeContent);

        $view = 'voyager::bread.read';

        if (view()->exists("voyager::$slug.read")) {
            $view = "voyager::$slug.read";
        }

        return Voyager::view($view, compact('dataType', 'dataTypeContent', 'isModelTranslatable', 'isSoftDeleted'));
    }

    //***************************************
    //                ______
    //               |  ____|
    //               | |__
    //               |  __|
    //               | |____
    //               |______|
    //
    //  Edit an item of our Data Type BR(E)AD
    //
    //****************************************

    public function edit(Request $request, $id)
    {
        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        if (strlen($dataType->model_name) != 0) {
            $model = app($dataType->model_name);

            // Use withTrashed() if model uses SoftDeletes and if toggle is selected
            if ($model && in_array(SoftDeletes::class, class_uses($model))) {
                $model = $model->withTrashed();
            }
            if ($dataType->scope && $dataType->scope != '' && method_exists($model, 'scope'.ucfirst($dataType->scope))) {
                $model = $model->{$dataType->scope}();
            }
            $dataTypeContent = call_user_func([$model, 'findOrFail'], $id);
        } else {
            // If Model doest exist, get data from table name
            $dataTypeContent = DB::table($dataType->name)->where('id', $id)->first();
        }

        foreach ($dataType->editRows as $key => $row) {
            $dataType->editRows[$key]['col_width'] = isset($row->details->width) ? $row->details->width : 100;
        }

        // If a column has a relationship associated with it, we do not want to show that field
        $this->removeRelationshipField($dataType, 'edit');

        // Check permission
        $this->authorize('edit', $dataTypeContent);

        // Check if BREAD is Translatable
        $isModelTranslatable = is_bread_translatable($dataTypeContent);

        $view = 'voyager::bread.edit-add';

        if (view()->exists("voyager::$slug.edit-add")) {
            $view = "voyager::$slug.edit-add";
        }

        return Voyager::view($view, compact('dataType', 'dataTypeContent', 'isModelTranslatable'));
    }

    // POST BR(E)AD
    public function update(Request $request, $id)
    {
        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        // Compatibility with Model binding.
        $id = $id instanceof Model ? $id->{$id->getKeyName()} : $id;

        $model = app($dataType->model_name);
        if ($dataType->scope && $dataType->scope != '' && method_exists($model, 'scope'.ucfirst($dataType->scope))) {
            $model = $model->{$dataType->scope}();
        }
        if ($model && in_array(SoftDeletes::class, class_uses($model))) {
            $data = $model->withTrashed()->findOrFail($id);
        } else {
            $data = call_user_func([$dataType->model_name, 'findOrFail'], $id);
        }

        // Check permission
        $this->authorize('edit', $data);

        // Validate fields with ajax
        $val = $this->validateBread($request->all(), $dataType->editRows, $dataType->name, $id)->validate();
        $this->insertUpdateData($request, $slug, $dataType->editRows, $data);

        foreach ($request->priceId as $key => $priceId) {
            $commandPrice = $this->commandPriceRepository->create([
                "clientId" => $data->clientId,
                "commandId" => $data->id,
                "priceId" => $priceId,
                "quantity" => array_key_exists($key, $request->quantity) && $request->quantity[$key] ? $request->quantity[$key] : 1
            ]);
        }

        event(new BreadDataUpdated($dataType, $data));

        return redirect()
        ->route("voyager.{$dataType->slug}.index")
        ->with([
            'message'    => __('voyager::generic.successfully_updated')." {$dataType->display_name_singular}",
            'alert-type' => 'success',
        ]);
    }

    //***************************************
    //
    //                   /\
    //                  /  \
    //                 / /\ \
    //                / ____ \
    //               /_/    \_\
    //
    //
    // Add a new item of our Data Type BRE(A)D
    //
    //****************************************

    public function create(Request $request)
    {
        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        // Check permission
        $this->authorize('add', app($dataType->model_name));

        $dataTypeContent = (strlen($dataType->model_name) != 0)
                            ? new $dataType->model_name()
                            : false;

        foreach ($dataType->addRows as $key => $row) {
            $dataType->addRows[$key]['col_width'] = $row->details->width ?? 100;
        }

        // If a column has a relationship associated with it, we do not want to show that field
        $this->removeRelationshipField($dataType, 'add');

        // Check if BREAD is Translatable
        $isModelTranslatable = is_bread_translatable($dataTypeContent);

        $view = 'voyager::bread.edit-add';

        if (view()->exists("voyager::$slug.edit-add")) {
            $view = "voyager::$slug.edit-add";
        }

        return Voyager::view($view, compact('dataType', 'dataTypeContent', 'isModelTranslatable'));
    }

    /**
     * POST BRE(A)D - Store data.
     *
     * @param \Illuminate\Http\Request $request
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function store(Request $request)
    {
        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        // Check permission
        $this->authorize('add', app($dataType->model_name));

        // Validate fields with ajax
        $val = $this->validateBread($request->all(), $dataType->addRows)->validate();
        $data = $this->insertUpdateData($request, $slug, $dataType->addRows, new $dataType->model_name());

        foreach ($request->priceId as $key => $priceId) {
            $commandPrice = $this->commandPriceRepository->create([
                "clientId" => $data->clientId,
                "commandId" => $data->id,
                "priceId" => $priceId,
                "quantity" => array_key_exists($key, $request->quantity) ? $request->quantity[$key] : 1
            ]);
        }

        event(new BreadDataAdded($dataType, $data));

        return redirect()
        ->route("voyager.{$dataType->slug}.index")
        ->with([
                'message'    => __('voyager::generic.successfully_added_new')." {$dataType->display_name_singular}",
                'alert-type' => 'success',
            ]);
    }

    //***************************************
    //                _____
    //               |  __ \
    //               | |  | |
    //               | |  | |
    //               | |__| |
    //               |_____/
    //
    //         Delete an item BREA(D)
    //
    //****************************************

    public function destroy(Request $request, $id)
    {
        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        // Check permission
        $this->authorize('delete', app($dataType->model_name));

        // Init array of IDs
        $ids = [];
        if (empty($id)) {
            // Bulk delete, get IDs from POST
            $ids = explode(',', $request->ids);
        } else {
            // Single item delete, get ID from URL
            $ids[] = $id;
        }
        foreach ($ids as $id) {
            $data = call_user_func([$dataType->model_name, 'findOrFail'], $id);

            $model = app($dataType->model_name);
            if (!($model && in_array(SoftDeletes::class, class_uses($model)))) {
                $this->cleanup($dataType, $data);
            }
        }

        $displayName = count($ids) > 1 ? $dataType->display_name_plural : $dataType->display_name_singular;

        $res = $data->destroy($ids);
        $data = $res
            ? [
                'message'    => __('voyager::generic.successfully_deleted')." {$displayName}",
                'alert-type' => 'success',
            ]
            : [
                'message'    => __('voyager::generic.error_deleting')." {$displayName}",
                'alert-type' => 'error',
            ];

        if ($res) {
            event(new BreadDataDeleted($dataType, $data));
        }

        return redirect()->route("voyager.{$dataType->slug}.index")->with($data);
    }

    public function restore(Request $request, $id)
    {
        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        // Check permission
        $this->authorize('delete', app($dataType->model_name));

        // Get record
        $model = call_user_func([$dataType->model_name, 'withTrashed']);
        if ($dataType->scope && $dataType->scope != '' && method_exists($model, 'scope'.ucfirst($dataType->scope))) {
            $model = $model->{$dataType->scope}();
        }
        $data = $model->findOrFail($id);

        $displayName = $dataType->display_name_singular;

        $res = $data->restore($id);
        $data = $res
            ? [
                'message'    => __('voyager::generic.successfully_restored')." {$displayName}",
                'alert-type' => 'success',
            ]
            : [
                'message'    => __('voyager::generic.error_restoring')." {$displayName}",
                'alert-type' => 'error',
            ];

        if ($res) {
            event(new BreadDataRestored($dataType, $data));
        }

        return redirect()->route("voyager.{$dataType->slug}.index")->with($data);
    }

    /**
     * Remove translations, images and files related to a BREAD item.
     *
     * @param \Illuminate\Database\Eloquent\Model $dataType
     * @param \Illuminate\Database\Eloquent\Model $data
     *
     * @return void
     */
    protected function cleanup($dataType, $data)
    {
        // Delete Translations, if present
        if (is_bread_translatable($data)) {
            $data->deleteAttributeTranslations($data->getTranslatableAttributes());
        }

        // Delete Images
        $this->deleteBreadImages($data, $dataType->deleteRows->where('type', 'image'));

        // Delete Files
        foreach ($dataType->deleteRows->where('type', 'file') as $row) {
            if (isset($data->{$row->field})) {
                foreach (json_decode($data->{$row->field}) as $file) {
                    $this->deleteFileIfExists($file->download_link);
                }
            }
        }

        // Delete media-picker files
        $dataType->rows->where('type', 'media_picker')->where('details.delete_files', true)->each(function ($row) use ($data) {
            $content = $data->{$row->field};
            if (isset($content)) {
                if (!is_array($content)) {
                    $content = json_decode($content);
                }
                if (is_array($content)) {
                    foreach ($content as $file) {
                        $this->deleteFileIfExists($file);
                    }
                } else {
                    $this->deleteFileIfExists($content);
                }
            }
        });
    }

    /**
     * Delete all images related to a BREAD item.
     *
     * @param \Illuminate\Database\Eloquent\Model $data
     * @param \Illuminate\Database\Eloquent\Model $rows
     *
     * @return void
     */
    public function deleteBreadImages($data, $rows, $single_image = null)
    {
        foreach ($rows as $row) {
            if ($data->{$row->field} != config('voyager.user.default_avatar')) {
                $this->deleteFileIfExists($data->{$row->field});
            }

            if (isset($row->details->thumbnails)) {
                foreach ($row->details->thumbnails as $thumbnail) {
                    $ext = explode('.', $data->{$row->field});
                    $extension = '.'.$ext[count($ext) - 1];

                    $path = str_replace($extension, '', $data->{$row->field});

                    $thumb_name = $thumbnail->name;

                    $this->deleteFileIfExists($path.'-'.$thumb_name.$extension);
                }
            }
        }

        if ($rows->count() > 0) {
            event(new BreadImagesDeleted($data, $rows));
        }
    }

    /**
     * Order BREAD items.
     *
     * @param string $table
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function order(Request $request)
    {
        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        // Check permission
        $this->authorize('edit', app($dataType->model_name));

        if (!isset($dataType->order_column) || !isset($dataType->order_display_column)) {
            return redirect()
            ->route("voyager.{$dataType->slug}.index")
            ->with([
                'message'    => __('voyager::bread.ordering_not_set'),
                'alert-type' => 'error',
            ]);
        }

        $model = app($dataType->model_name);
        if ($model && in_array(SoftDeletes::class, class_uses($model))) {
            $model = $model->withTrashed();
        }
        $results = $model->orderBy($dataType->order_column, $dataType->order_direction)->get();

        $display_column = $dataType->order_display_column;

        $dataRow = Voyager::model('DataRow')->whereDataTypeId($dataType->id)->whereField($display_column)->first();

        $view = 'voyager::bread.order';

        if (view()->exists("voyager::$slug.order")) {
            $view = "voyager::$slug.order";
        }

        return Voyager::view($view, compact(
            'dataType',
            'display_column',
            'dataRow',
            'results'
        ));
    }

    public function update_order(Request $request)
    {
        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        // Check permission
        $this->authorize('edit', app($dataType->model_name));

        $model = app($dataType->model_name);

        $order = json_decode($request->input('order'));
        $column = $dataType->order_column;
        foreach ($order as $key => $item) {
            if ($model && in_array(SoftDeletes::class, class_uses($model))) {
                $i = $model->withTrashed()->findOrFail($item->id);
            } else {
                $i = $model->findOrFail($item->id);
            }
            $i->$column = ($key + 1);
            $i->save();
        }
    }

    public function action(Request $request)
    {
        $slug = $this->getSlug($request);
        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        $action = new $request->action($dataType, null);

        return $action->massAction(explode(',', $request->ids), $request->headers->get('referer'));
    }

    /**
     * Get BREAD relations data.
     *
     * @param Request $request
     *
     * @return mixed
     */
    public function relation(Request $request)
    {
        $slug = $this->getSlug($request);
        $page = $request->input('page');
        $on_page = 50;
        $search = $request->input('search', false);
        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        $rows = $request->input('method', 'add') == 'add' ? $dataType->addRows : $dataType->editRows;
        foreach ($rows as $key => $row) {
            if ($row->field === $request->input('type')) {
                $options = $row->details;
                $skip = $on_page * ($page - 1);

                // If search query, use LIKE to filter results depending on field label
                if ($search) {
                    $total_count = app($options->model)->where($options->label, 'LIKE', '%'.$search.'%')->count();
                    $relationshipOptions = app($options->model)->take($on_page)->skip($skip)
                        ->where($options->label, 'LIKE', '%'.$search.'%')
                        ->get();
                } else {
                    $total_count = app($options->model)->count();
                    $relationshipOptions = app($options->model)->take($on_page)->skip($skip)->get();
                }

                $results = [];
                foreach ($relationshipOptions as $relationshipOption) {
                    $results[] = [
                        'id'   => $relationshipOption->{$options->key},
                        'text' => $relationshipOption->{$options->label},
                    ];
                }

                return response()->json([
                    'results'    => $results,
                    'pagination' => [
                        'more' => ($total_count > ($skip + $on_page)),
                    ],
                ]);
            }
        }

        // No result found, return empty array
        return response()->json([], 404);
    }
    
    
    public function resetAllCommands() {
        
        $preserves = [5720, 5691, 5674, 5673, 5672, 5671];
        $commands = \DB::table('Command')->select('id')->whereNull('delivery_state_id')->where('canceled', 0)->whereNotIn('pos_command_id', $preserves)->pluck('id');
        
        foreach ($commands as $commandId) {

            $command = $this->commandRepository->find($commandId);
            $commandLines = [];
            $deliveryCompany = null;
            $command->canceled = true;
            $command->save();
            
            $client = new \GuzzleHttp\Client([
                'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
                'defaults' => [
                    'exceptions' => false
                ],
                'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
            ]);
    
            $commandDetails = json_decode($client->post(config('urls.command-details'), [
                'json' => [
                    'cmdClientIds' =>[ $command->pos_command_id]
                ]
            ])->getBody());
            
            $data = [
                'client_id' => $commandDetails->CmdClient[0]->client_id,
                'stateTag'  => 'validee'
            ];
    
            $local_key = $commandDetails->CmdClient[0]->local_key;
           
            $commandLines = $commandDetails->CmdPrdCnsPrcpl;

            foreach ($command->lines as $line) {
                try{
                    $product = $line->price->products->first();
                }catch(\Exception $erorr) {
                    continue;
                }
                $commandPrice = $this->commandPriceRepository->getCommandPrice($command->id, $line->price->id);
                $productStock = StockOperationProduct::where('productId', $product->id)->first();
                $productStock->quantity +=  $commandPrice->quantity;
                $productStock->save();
            }
    
            SyncCommandsToPos::dispatch($command, $commandLines, $local_key, $deliveryCompany, $data)->onQueue('sync-commands');
        }
        
        return "Succées";
    }
    
    public function retryFailedJobs(Request $request) {
        
        Artisan::call('queue:retry', [
            'id' => 'all',
        ]);
        
        Artisan::call('command:syncCommands');
        
        return back()->with([
            'message'    => "Commandes rafraîchi avec succès",
            'alert-type' => 'success',
        ]);
    }
    
    public function handleCommandV2(Request $request, $commandId)
    {  
        
        $command = $this->commandRepository->find($commandId);

        $commandLines = [];
        $deliveryCompany = null;
        $state = $request->query('state');
        
        if($state =='cancele') $command->canceled = true;
        else $command->validated = true;
        
        $command->save();
        
        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);

        $commandDetails = json_decode($client->post(config('urls.command-details'), [
            'json' => [
                'cmdClientIds' =>[ $command->pos_command_id]
            ]
        ])->getBody());
        
        $data = [
            'client_id' => $commandDetails->CmdClient[0]->client_id,
            'stateTag'  => $state == 'cancele' ? 'validee' : null
        ];

        $local_key = $commandDetails->CmdClient[0]->local_key;
       
        $commandLines = $commandDetails->CmdPrdCnsPrcpl;

        //sate validee mean the we gonna cancel commande
        if($state == 'cancele'){
            foreach ($command->lines as $line) {
                try{
                    $product = $line->price->products->first();
                }catch(\Exception $erorr) {
                    continue;
                }
                $commandPrice = $this->commandPriceRepository->getCommandPrice($command->id, $line->price->id);
                $productStock = StockOperationProduct::where('productId', $product->id)->first();
                $productStock->quantity +=  $commandPrice->quantity;
                $productStock->save();
            }
        }

        SyncCommandsToPos::dispatch($command, $commandLines, $local_key, $deliveryCompany, $data)->onQueue('sync-commands');
        
        $word = $state == "cancele" ? "annulé" : "validé";
        
        return [
            'message' => "Commande $word avec succès",
        ];
        
    }
    
    
    

    public function handleCommand(Request $request, $commandId)
    {  
        
        $command = $this->commandRepository->find($commandId);

        $commandLines = [];
        $deliveryCompany = null;
        $state = $request->query('state');
        
        if($state =='cancele') $command->canceled = true;
        else $command->validated = true;
        
        $command->save();
        
        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);

        $commandDetails = json_decode($client->post(config('urls.command-details'), [
            'json' => [
                'cmdClientIds' =>[ $command->pos_command_id]
            ]
        ])->getBody());
        
        $data = [
            'client_id' => $commandDetails->CmdClient[0]->client_id,
            'stateTag'  => $state == 'cancele' ? 'validee' : null
        ];

        $local_key = $commandDetails->CmdClient[0]->local_key;
       
        $commandLines = $commandDetails->CmdPrdCnsPrcpl;

        //sate validee mean the we gonna cancel commande
        if($state == 'cancele'){
            foreach ($command->lines as $line) {
                try{
                    $product = \App\Models\Product::find(\DB::table('CombinedProduct')->where('price_id', $line->priceId)->first()->product_id);
                    $commandPrice = $this->commandPriceRepository->getCommandPrice($command->id, $line->priceId);
                    $productStock = StockOperationProduct::where('productId', $product->id)->first();
                    $productStock->quantity +=  $commandPrice->quantity;
                    $productStock->save();

                }catch(\Exception $erorr) {
                    continue;
                }
            }
        }

        SyncCommandsToPos::dispatch($command, $commandLines, $local_key, $deliveryCompany, $data)->onQueue('sync-commands');
        
        $word = $state == "cancele" ? "annulé" : "validé";
        
        return back()->with([
            'message'    => "Commande $word avec succès",
            'alert-type' => 'success',
        ]);
    }
    
    public function handleTestCommand(Request $request, $commandId)
    { 
        
        $command = $this->commandRepository->find($commandId);

        $commandLines = [];

        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);

        $commandDetails = json_decode($client->post(config('urls.command-details'), [
            'json' => [
                'cmdClientIds' =>[ $command->pos_command_id]
            ]
        ])->getBody());
        

        $local_key = $commandDetails->CmdClient[0]->local_key;
       
        $commandLines = $commandDetails->CmdPrdCnsPrcpl;
        $errors = [];

        foreach ($command->lines as $line) {
            try{
                $product = \App\Models\Product::find(\DB::table('CombinedProduct')->where('price_id', $line->priceId)->first()->product_id);
                $commandPrice = $this->commandPriceRepository->getCommandPrice($command->id, $line->priceId);
                $productStock = StockOperationProduct::where('productId', $product->id)->first();
                $productStock->quantity +=  $commandPrice->quantity;
                $productStock->save();
                
            }catch(\Exception $erorr) {
                continue;
            }
        }
        
        dd($errors);
    }
}
